/*-
 *  Copyright (c) 2003  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *  $Ringlet: cs/net/ProxyUtils/ProxyUtils/MatchWebProxy.cs,v 1.1.1.1 2003/04/03 06:32:26 roam Exp $
 */

using System;
using System.Net;

namespace Ringlet.Net.ProxyUtils
{
	/// <summary>
	/// The actual proxy class which uses the ProxyMatchMaker
	/// to determine the proxy to be used for each request.
	/// </summary>
	public class MatchWebProxy : IWebProxy
	{
		/// <summary>
		/// The credentials used for the request.
		/// </summary>
		/// <remarks>FIXME: pass them further down?</remarks>
		ICredentials creds;
		/// <summary>
		/// The match-maker to use.
		/// </summary>
		ProxyMatchMaker match;

		/// <summary>
		/// Default constructor.
		/// </summary>
		public MatchWebProxy()
		{
			creds = null;
			match = null;
		}

		/// <summary>
		/// A constructor specifying a match-maker.
		/// </summary>
		/// <param name="nmatch"></param>
		public MatchWebProxy(ProxyMatchMaker nmatch)
		{
			creds = null;
			match = nmatch;
		}

		/// <summary>
		/// Get or set the <see>creds</see> member variable.
		/// </summary>
		public ICredentials Credentials
		{
			get
			{
				return (creds);
			}

			set
			{
				creds = value;
			}
		}

		/// <summary>
		/// Query the match-maker for a proxy to use for
		/// a specific URI.
		/// </summary>
		/// <param name="uri">The URI to examine</param>
		/// <returns>The IWebProxy to use, or 'null' if not found.</returns>
		protected IWebProxy FindProxy(Uri uri)
		{
			if (match == null)
				return (null);
			ProxyMatchRule rule = match.Ruleset.Match(uri);
			if (rule == null)
				return (null);
			return (rule.Proxy);
		}

		/// <summary>
		/// Overridden.
		/// Determine if the proxy should be bypassed for
		/// a particular URI.
		/// </summary>
		/// <param name="uri">The URI to examine</param>
		/// <returns>True if the proxy should be bypassed.</returns>
		public bool IsBypassed(Uri uri)
		{
			IWebProxy proxy = FindProxy(uri);
			
			if (proxy == null)
				return (true);
			else
				return (proxy.IsBypassed(uri));
		}

		/// <summary>
		/// Overridden.
		/// Determine the proxy to use for a specific URI.
		/// </summary>
		/// <param name="uri">The URI to examine</param>
		/// <returns>The proxy to use, or the URI itself if none.</returns>
		public Uri GetProxy(Uri uri)
		{
			IWebProxy proxy = FindProxy(uri);

			if (proxy == null)
				return (uri);
			else
				return (proxy.GetProxy(uri));
		}
	}
}
