/*-
 *  Copyright (c) 2003  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *  $Ringlet: cs/net/ProxyUtils/ProxyUtils/ProxyMatchRule.cs,v 1.1.1.1 2003/04/03 06:32:26 roam Exp $
 */

using System;
using System.Net;
using System.Text.RegularExpressions;

namespace Ringlet.Net.ProxyUtils
{
	/// <summary>
	/// Construct a rule to match the request URI's against,
	/// and perform the actual matching.
	/// </summary>
	/// <remarks>FIXME: add a constructor from a text string</remarks>
	public class ProxyMatchRule
	{
		/// <summary>
		/// The rule index in the ruleset it belongs to.
		/// </summary>
		protected Int16 index;
		/// <summary>
		/// How to match the pattern against the request URI.
		/// </summary>
		protected ProxyMatchMode mode;
		/// <summary>
		/// Which part of the request URI to match the pattern against.
		/// </summary>
		protected ProxyMatchComponent comp;
		/// <summary>
		/// The actual pattern to match against.
		/// </summary>
		protected string pattern;
		/// <summary>
		/// The proxy to use on success.
		/// </summary>
		protected IWebProxy proxy;
		/// <summary>
		/// The regular expression used internally for matching.
		/// </summary>
		protected Regex regex;

		/// <summary>
		/// A constructor helper: initialize all member
		/// variables to sensible (but unusable) defaults.
		/// </summary>
		protected void SetDefaults()
		{
			index = 0;
			mode = ProxyMatchMode.Exact;
			comp = ProxyMatchComponent.URIWhole;
			pattern = null;
			proxy = null;
			regex = null;
		}

		/// <summary>
		/// Default constructor: uses <see cref="SetDefaults"/>
		/// to initialize the match-maker to a sensible, yet
		/// unusable state.
		/// </summary>
		public ProxyMatchRule()
		{
			SetDefaults();
		}

		/// <summary>
		/// Match an exact request URI.
		/// </summary>
		/// <param name="npattern">The URI to match.</param>
		/// <param name="nproxy">The proxy to use.</param>
		public ProxyMatchRule(string npattern, IWebProxy nproxy)
		{
			SetDefaults();
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Match an exact request URI, and place at the specified
		/// ruleset position.
		/// </summary>
		/// <param name="nindex">Ruleset position</param>
		/// <param name="npattern">The request URI to match</param>
		/// <param name="nproxy">The proxy to use</param>
		public ProxyMatchRule(Int16 nindex, string npattern,
			IWebProxy nproxy)
		{
			SetDefaults();
			index = nindex;
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Match a given component of the URI using a given mode.
		/// </summary>
		/// <param name="nmode">How to match</param>
		/// <param name="ncomp">What to match</param>
		/// <param name="npattern">Match pattern</param>
		/// <param name="nproxy">The proxy to use</param>
		public ProxyMatchRule(ProxyMatchMode nmode,
			ProxyMatchComponent ncomp, string npattern, IWebProxy nproxy)
		{
			SetDefaults();
			mode = nmode;
			comp = ncomp;
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Match a given component of the URI using a given mode.
		/// </summary>
		/// <param name="nindex">Ruleset position</param>
		/// <param name="nmode">How to match</param>
		/// <param name="ncomp">What to match</param>
		/// <param name="npattern">Match pattern</param>
		/// <param name="nproxy">The proxy to use</param>
		public ProxyMatchRule(Int16 nindex, ProxyMatchMode nmode,
			ProxyMatchComponent ncomp, string npattern, IWebProxy nproxy)
		{
			SetDefaults();
			index = nindex;
			mode = nmode;
			comp = ncomp;
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Match the URI using a given mode.
		/// </summary>
		/// <param name="nmode">How to match</param>
		/// <param name="npattern">Match pattern</param>
		/// <param name="nproxy">The proxy to use</param>
		public ProxyMatchRule(ProxyMatchMode nmode,
			string npattern, IWebProxy nproxy)
		{
			SetDefaults();
			mode = nmode;
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Match the URI using a given mode.
		/// </summary>
		/// <param name="nindex">Ruleset position</param>
		/// <param name="nmode">How to match</param>
		/// <param name="npattern">Match pattern</param>
		/// <param name="nproxy">The proxy to use</param>
		public ProxyMatchRule(Int16 nindex, ProxyMatchMode nmode,
			string npattern, IWebProxy nproxy)
		{
			index = nindex;
			mode = nmode;
			comp = ProxyMatchComponent.URIWhole;
			pattern = npattern;
			proxy = nproxy;
			CompilePattern();
		}

		/// <summary>
		/// Get or set the rule's position in the ruleset.
		/// </summary>
		public Int16 Index
		{
			get
			{
				return (index);
			}

			set
			{
				index = value;
			}
		}

		/// <summary>
		/// Get or set the rule's matching mode.
		/// </summary>
		public ProxyMatchMode Mode
		{
			get
			{
				return (mode);
			}

			set
			{
				mode = value;
				CompilePattern();
			}
		}

		/// <summary>
		/// Get or set the part of the request URI to match against.
		/// </summary>
		public ProxyMatchComponent Component
		{
			get
			{
				return (comp);
			}

			set
			{
				comp = value;
			}
		}

		/// <summary>
		/// Get or set the actual pattern to match against.
		/// </summary>
		public string Pattern
		{
			get
			{
				return (pattern);
			}

			set
			{
				pattern = value;
				CompilePattern();
			}
		}

		/// <summary>
		/// Get or set the proxy to use on success.
		/// </summary>
		public IWebProxy Proxy
		{
			get
			{
				return (proxy);
			}

			set
			{
				proxy = value;
			}
		}

		/// <summary>
		/// Escape characters which have a special meaning in
		/// regular expressions.
		/// </summary>
		/// <param name="s">The string to escape</param>
		/// <param name="chars">The characters to escape,
		/// or 'null' for all the special characters</param>
		/// <returns>The string with all special characters
		/// suitably escaped.</returns>
		protected string RegexFix(string s, string chars)
		{
			string res = s;

			if (chars == null)
				chars = "\\.?*";

			if (chars.IndexOf('\\') >= 0)
				res = res.Replace("\\", "\\\\");
			if (chars.IndexOf('.') >= 0)
				res = res.Replace(".", "\\.");
			if (chars.IndexOf('*') >= 0)
				res = res.Replace("*", "\\*");
			if (chars.IndexOf('?') >= 0)
				res = res.Replace("?", "\\?");
			return (res);
		}

		/// <summary>
		/// Build a suitable regular expression to match against,
		/// depending on the mode and component specified.
		/// </summary>
		protected void CompilePattern()
		{
			string pat = pattern;
			
			regex = null;
			switch (mode)
			{
				case ProxyMatchMode.Exact:
					return;

				case ProxyMatchMode.StartsWith:
					return;

				case ProxyMatchMode.Contains:
					return;
					
				case ProxyMatchMode.EndsWith:
					return;
					
				case ProxyMatchMode.Regexp:
					break;

				case ProxyMatchMode.WildMat:
					pat = RegexFix(pat, "\\.");
					pat = pat.Replace("?", ".").Replace("*", ".*");
					break;

				case ProxyMatchMode.SQLLike:
					pat = RegexFix(pat, null);
					pat = pat.Replace("_", ".").Replace("%", ".*");
					break;

				default:
					throw (new ApplicationException(
					    "Unsupported ProxyMatchMode " +
					    mode.ToString()));
			}
			
			regex = new Regex(pat,
			    RegexOptions.Compiled | RegexOptions.IgnoreCase);
		}

		/// <summary>
		/// Perform the actual matching of an URI against
		/// the specified constraints.
		/// </summary>
		/// <param name="uri">The request URI to match</param>
		/// <returns>True if the URI matches.</returns>
		public bool Match(Uri uri)
		{
			string s;
			
			switch (comp)
			{
				case ProxyMatchComponent.URIWhole:
					s = uri.AbsoluteUri;
					break;

				case ProxyMatchComponent.URIScheme:
					s = uri.Scheme;
					break;

				case ProxyMatchComponent.URIServer:
					if (uri.UserInfo != null &&
						uri.UserInfo != "")
						s = uri.UserInfo + "@";
					else
						s = "";
					s += uri.Authority;
					break;

				case ProxyMatchComponent.URIServerHost:
					s = uri.Host;
					break;

				case ProxyMatchComponent.URIServerPassword:
					if (uri.UserInfo == null ||
					    uri.UserInfo == "" ||
					    uri.UserInfo.IndexOf(':') < 0)
						s = "";
					else
						s = uri.UserInfo.Substring(
						    uri.UserInfo.IndexOf(':') + 1);
					break;

				case ProxyMatchComponent.URIServerPort:
					s = uri.Port.ToString();
					break;

				case ProxyMatchComponent.URIServerUser:
					if (uri.UserInfo == null ||
					    uri.UserInfo == "")
						s = "";
					else if (uri.UserInfo.IndexOf(':') < 0)
						s = uri.UserInfo;
					else
						s = uri.UserInfo.Substring(0,
						    uri.UserInfo.IndexOf(':'));
					break;
					
				case ProxyMatchComponent.URIPath:
					s = uri.AbsolutePath;
					break;

				case ProxyMatchComponent.URIQuery:
					s = uri.Query;
					break;

				default:
					throw (new ApplicationException(
					    "Unsupported match mode: " + comp.ToString()));
			}

			switch (mode)
			{
				case ProxyMatchMode.Exact:
					return (s == pattern);

				case ProxyMatchMode.StartsWith:
					return (s.StartsWith(pattern));

				case ProxyMatchMode.EndsWith:
					return (s.EndsWith(pattern));

				case ProxyMatchMode.Contains:
					return (s.IndexOf(pattern) >= 0);

				default:
					if (regex == null)
						return (false);
					return (regex.Match(s).Success);
			}
		}
	}
}
