/*-
 *  Copyright (c) 2003  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *  $Ringlet: cs/net/ProxyUtils/ProxyUtils/ProxyMatchRuleset.cs,v 1.1.1.1 2003/04/03 06:32:26 roam Exp $
 */

using System;
using System.Collections;

namespace Ringlet.Net.ProxyUtils
{
	/// <summary>
	/// An ordered set of ProxyMatchRule's to match against.
	/// </summary>
	public class ProxyMatchRuleset
	{
		/// <summary>
		/// The list of rules themselves.
		/// </summary>
		protected ArrayList rules;
		/// <summary>
		/// The index of the last added rule, if
		/// automatic rule numbering is requested.
		/// </summary>
		protected Int16 lastIndex;

		/// <summary>
		/// Default constructor.
		/// </summary>
		public ProxyMatchRuleset()
		{
			rules = new ArrayList();
			lastIndex = 0;
		}

		/// <summary>
		/// Add a rule to the ruleset, placing it at
		/// a suitable position, and autogenerating
		/// an index if the rule does not specify one.
		/// </summary>
		/// <param name="rule">The rule to add</param>
		/// <returns>The rule placed in the ruleset.</returns>
		public ProxyMatchRule Add(ProxyMatchRule rule)
		{
			int i;

			// Auto-generate the index?
			if (rule.Index == 0)
				rule.Index = (Int16)(lastIndex + 100);
			// Store the index for later use.
			lastIndex = rule.Index;

			// Find the least-numbered rule *after* ours.
			for (i = 0; i < rules.Count; i++)
				if (((ProxyMatchRule)rules[i]).Index > rule.Index)
					break;
			if (i == rules.Count)
				rules.Add(rule);
			else
				rules.Insert(i, rule);
			return (rule);
		}

		/// <summary>
		/// Match a request URI against the ordered ruleset.
		/// </summary>
		/// <param name="uri">The URI to match</param>
		/// <returns>The matching rule, or null if none.</returns>
		public ProxyMatchRule Match(Uri uri)
		{
			int i;

			for (i = 0; i < rules.Count; i++)
			{
				ProxyMatchRule r = rules[i] as ProxyMatchRule;
				if (r != null && r.Match(uri))
					return (r);
			}
			return (null);
		}
	}
}
